@@ -339,25 +339,12 @@ pygobject_free(PyObject *op)
 
 /* ---------------- PyGObject methods ----------------- */
 
-static void
-parameter_list_free (GParameter *parameters, guint n_parameters)
-{
-	gint i;
-	for (i = 0; i < n_parameters; i ++){
-		g_free ((char *)parameters[i].name);
-		g_value_unset (&(parameters[i].value));
-	}
-	g_free (parameters);
-}
-
 static int
 pygobject_init(PyGObject *self, PyObject *args, PyObject *kwargs)
 {
     GType object_type;
-    guint n_parameters = 0;
-    GParameter *parameters = NULL;
-    PyObject *key, *item;
-    gint pos = 0;
+    guint n_params = 0, i;
+    GParameter *params = NULL;
     GObjectClass *class;
 
     if (!PyArg_ParseTuple(args, ":GObject.__init__", &object_type))
@@ -367,63 +354,63 @@ pygobject_init(PyGObject *self, PyObject *args, PyObject *kwargs)
     if (!object_type)
 	return -1;
 
+    if (G_TYPE_IS_ABSTRACT(object_type)) {
+	PyErr_Format(PyExc_TypeError, "cannot create instance of abstract "
+		     "(non-instantiable) type `%s'", g_type_name(object_type));
+	return -1;
+    }
+
     if ((class = g_type_class_ref (object_type)) == NULL) {
 	PyErr_SetString(PyExc_TypeError,
 			"could not get a reference to type class");
 	return -1;
     }
 
+    if (kwargs) {
+	int pos = 0;
+	PyObject *key;
+	PyObject *value;
 
-    while (kwargs && PyDict_Next(kwargs, &pos, &key, &item)) {
-	    gchar *param_name;
+	params = g_new0(GParameter, PyDict_Size(kwargs));
+	while (PyDict_Next (kwargs, &pos, &key, &value)) {
 	    GParamSpec *pspec;
-	    GValue value = { 0, };
+	    gchar *key_str = PyString_AsString(key);
 
-	    param_name = PyString_AsString(key);
-	    pspec = g_object_class_find_property(class, param_name);
-
-	    if (pspec == NULL) {
-		    gchar buf[128];
-		    
-		    g_snprintf(buf, sizeof(buf),
-			       "Unknown parameter '%s' used in type initialization arguments",
-			       param_name);
-		    PyErr_SetString(PyExc_AttributeError, buf);
-		    parameter_list_free (parameters, n_parameters);
-		    g_type_class_unref(class);
-		    return -1;
+	    pspec = g_object_class_find_property (class, key_str);
+	    if (!pspec) {
+		PyErr_Format(PyExc_TypeError,
+			     "gobject `%s' doesn't support property `%s'",
+			     g_type_name(object_type), key_str);
+		goto cleanup;
 	    }
-	    g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(pspec));
-	    if (pyg_value_from_pyobject(&value, item) < 0) {
-		    PyErr_SetString(PyExc_TypeError,
-			"could not convert argument to correct param type");
-		    parameter_list_free (parameters, n_parameters);
-		    g_type_class_unref(class);
-		    return -1;
+	    g_value_init(&params[n_params].value,
+			 G_PARAM_SPEC_VALUE_TYPE(pspec));
+	    if (pyg_value_from_pyobject(&params[n_params].value, value)) {
+		PyErr_Format(PyExc_TypeError,
+			     "could not convert value for property `%s'",
+			     key_str);
+		goto cleanup;
 	    }
-
-	    n_parameters ++;
-	    if (parameters == NULL) {
-		    parameters = g_new (GParameter, 1);
-	    } else {
-		    parameters = g_renew (GParameter, parameters, n_parameters);
-	    }
-	    parameters[n_parameters - 1].name = g_strdup (param_name);
-	    parameters[n_parameters - 1].value = value;
+	    params[n_params].name = g_strdup(key_str);
+	    n_params++;
 	}
-
-    self->obj = g_object_newv(object_type, n_parameters, parameters);
-    parameter_list_free (parameters, n_parameters);
-
-    g_type_class_unref(class);
-
-    if (!self->obj) {
-	PyErr_SetString(PyExc_RuntimeError, "could not create object");
-	return -1;
     }
-    pygobject_register_wrapper((PyObject *)self);
 
-    return 0;
+    self->obj = g_object_newv(object_type, n_params, params);
+    if (self->obj)
+	pygobject_register_wrapper((PyObject *)self);
+    else
+	PyErr_SetString (PyExc_RuntimeError, "could not create object");
+	   
+ cleanup:
+    for (i = 0; i < n_params; i++) {
+	g_free((gchar *) params[i].name);
+	g_value_unset(&params[i].value);
+    }
+    g_free(params);
+    g_type_class_unref(class);
+    
+    return (self->obj) ? 0 : -1;
 }
 
 static PyObject *
@@ -670,7 +657,7 @@ pygobject_connect_object(PyGObject *self, PyObject *args)
     extra_args = PySequence_GetSlice(args, 3, len);
     if (extra_args == NULL)
 	return NULL;
-    closure = pyg_closure_new(callback, extra_args, NULL);
+    closure = pyg_closure_new(callback, extra_args, object);
     pygobject_watch_closure((PyObject *)self, closure);
     handlerid = g_signal_connect_closure_by_id(self->obj, sigid, detail,
 					       closure, FALSE);
@@ -712,7 +699,7 @@ pygobject_connect_object_after(PyGObject *self, PyObject *args)
     extra_args = PySequence_GetSlice(args, 3, len);
     if (extra_args == NULL)
 	return NULL;
-    closure = pyg_closure_new(callback, extra_args, NULL);
+    closure = pyg_closure_new(callback, extra_args, object);
     pygobject_watch_closure((PyObject *)self, closure);
     handlerid = g_signal_connect_closure_by_id(self->obj, sigid, detail,
 					       closure, TRUE);
@@ -863,6 +850,12 @@ pygobject_chain_from_overridden(PyGObject *self, PyObject *args)
     GValue *params, ret = { 0, };
 
     ihint = g_signal_get_invocation_hint(self->obj);
+    if (!ihint) {
+	PyErr_SetString(PyExc_TypeError, "could not find signa invocation "
+			"information for this object.");
+	return NULL;
+    }
+
     signal_id = ihint->signal_id;
     name = g_signal_name(signal_id);
 
